# Generated by roxygen2: do not edit by hand

S3method(predict,ww_area_of_applicability)
S3method(print,ww_area_of_applicability)
S3method(ww_agreement_coefficient,data.frame)
S3method(ww_area_of_applicability,data.frame)
S3method(ww_area_of_applicability,default)
S3method(ww_area_of_applicability,formula)
S3method(ww_area_of_applicability,matrix)
S3method(ww_area_of_applicability,recipe)
S3method(ww_area_of_applicability,rset)
S3method(ww_global_geary_c,data.frame)
S3method(ww_global_geary_pvalue,data.frame)
S3method(ww_global_moran_i,data.frame)
S3method(ww_global_moran_pvalue,data.frame)
S3method(ww_local_geary_c,data.frame)
S3method(ww_local_geary_pvalue,data.frame)
S3method(ww_local_getis_ord_g,data.frame)
S3method(ww_local_getis_ord_g_pvalue,data.frame)
S3method(ww_local_moran_i,data.frame)
S3method(ww_local_moran_pvalue,data.frame)
S3method(ww_multi_scale,SpatRaster)
S3method(ww_multi_scale,sf)
S3method(ww_systematic_agreement_coefficient,data.frame)
S3method(ww_systematic_mpd,data.frame)
S3method(ww_systematic_mse,data.frame)
S3method(ww_systematic_rmpd,data.frame)
S3method(ww_systematic_rmse,data.frame)
S3method(ww_unsystematic_agreement_coefficient,data.frame)
S3method(ww_unsystematic_mpd,data.frame)
S3method(ww_unsystematic_mse,data.frame)
S3method(ww_unsystematic_rmpd,data.frame)
S3method(ww_unsystematic_rmse,data.frame)
S3method(ww_willmott_d,data.frame)
S3method(ww_willmott_d1,data.frame)
S3method(ww_willmott_dr,data.frame)
export(ww_agreement_coefficient)
export(ww_agreement_coefficient_vec)
export(ww_area_of_applicability)
export(ww_build_neighbors)
export(ww_build_weights)
export(ww_global_geary_c)
export(ww_global_geary_c_vec)
export(ww_global_geary_pvalue)
export(ww_global_geary_pvalue_vec)
export(ww_global_moran_i)
export(ww_global_moran_i_vec)
export(ww_global_moran_pvalue)
export(ww_global_moran_pvalue_vec)
export(ww_local_geary_c)
export(ww_local_geary_c_vec)
export(ww_local_geary_pvalue)
export(ww_local_geary_pvalue_vec)
export(ww_local_getis_ord_g)
export(ww_local_getis_ord_g_pvalue)
export(ww_local_getis_ord_g_pvalue_vec)
export(ww_local_getis_ord_g_vec)
export(ww_local_moran_i)
export(ww_local_moran_i_vec)
export(ww_local_moran_pvalue)
export(ww_local_moran_pvalue_vec)
export(ww_make_point_neighbors)
export(ww_make_polygon_neighbors)
export(ww_multi_scale)
export(ww_systematic_agreement_coefficient)
export(ww_systematic_agreement_coefficient_vec)
export(ww_systematic_mpd)
export(ww_systematic_mpd_vec)
export(ww_systematic_mse)
export(ww_systematic_mse_vec)
export(ww_systematic_rmpd)
export(ww_systematic_rmpd_vec)
export(ww_systematic_rmse)
export(ww_systematic_rmse_vec)
export(ww_unsystematic_agreement_coefficient)
export(ww_unsystematic_agreement_coefficient_vec)
export(ww_unsystematic_mpd)
export(ww_unsystematic_mpd_vec)
export(ww_unsystematic_mse)
export(ww_unsystematic_mse_vec)
export(ww_unsystematic_rmpd)
export(ww_unsystematic_rmpd_vec)
export(ww_unsystematic_rmse)
export(ww_unsystematic_rmse_vec)
export(ww_willmott_d)
export(ww_willmott_d1)
export(ww_willmott_d1_vec)
export(ww_willmott_d_vec)
export(ww_willmott_dr)
export(ww_willmott_dr_vec)
importFrom(rlang,.data)
importFrom(rlang,.env)
importFrom(rlang,enquo)
importFrom(stats,complete.cases)
importFrom(stats,na.fail)
importFrom(stats,predict)
importFrom(yardstick,new_numeric_metric)
importFrom(yardstick,yardstick_any_missing)
importFrom(yardstick,yardstick_remove_missing)
